library(R2jags);library(survival)
setwd("C:/Users/TMurray/Documents/Thesis")

#JAGS Code to fit our covariate adjusted model 
write("data{
 ## Data Construction for the Historical Likelihood ##
	  for (i0 in 1:n0) {
	         for (k in 1:K) {
	            # indicates event-time in interval k
			zed0[i0,k] <- delta0[i0]*step(y0[i0] - kappa[k] - 0.00001)*step(kappa[k+1] - y0[i0]);   
				
		      # length of overlap of y0[i0] with interval k
		      theta0[i0,k] <- (min(y0[i0], kappa[k+1]) - kappa[k])*step(y0[i0] - kappa[k]);      
			
	         }
	   } 
 ## Data Construction for the Current Likelihood ##
	  for (i in 1:n) {
	         for (k in 1:K) {
	            # indicates event-time in interval k
			zed[i,k] <- delta[i]*step(y[i] - kappa[k] - 0.00001)*step(kappa[k+1] - y[i]);   
				
		      # length of overlap of y[i] with interval k
		      theta[i,k] <- (min(y[i], kappa[k+1]) - kappa[k])*step(y[i] - kappa[k]);      
	         }
	   } 
}

model{ 
 ## Historical Likelihoods ##
	  for (i0 in 1:n0) {
	         for (k in 1:K) {
		    # define the likelihood
			mu0[i0,k] <- theta0[i0,k]*exp(alpha0[k]+inprod(beta0[],X0[i0,]))
			zed0[i0,k] ~ dpois(mu0[i0,k]);
	         }
	   } 

 ## Current Likelihoods ##
	  for (i in 1:n) {
	         for (k in 1:K) {			
		    # define the likelihood
			#Use for gamma constant over time
			 mu[i,k] <- theta[i,k]*exp(alpha[k]+inprod(beta[],X[i,])+inprod(gamma[],Z[i,]))
			#Use for time-dependent gamma
			 #mu[i,k] <- theta[i,k]*exp(alpha[k]+inprod(beta[],X[i,])+inprod(gamma[k,],Z[i,]))
			zed[i,k] ~ dpois(mu[i,k]);
	         }
	   } 

	#Vague Priors for Treatment Effect and Historical Log-Hazards
	#Commensurate Priors on Current Log-Hazards
	  for (j in 1:p){
		 beta0[j] ~ dnorm(0,0.0001);
		 beta[j] ~ dnorm(beta0[j],tau[j]);
	  }
	  for (j in 1:q){
		 #Use for constant gamma
		  gamma[j] ~ dnorm(0,0.0001)
		 #Use for time-dependent gamma
		  #gamma[1,j] ~ dnorm(0,0.0001)
		  xi[j] <- 1/(sigmaX[j]*sigmaX[j]);
		  sigmaX[j] ~ dunif(0.01,100);
	  }
	  	 alpha0[1] ~ dnorm(0,0.0001);
	  	 alpha[1] ~ dnorm(alpha0[1], tau[p+1]);
		 v <- tau[p+1]/spike[2];
	  for (k in 2:K) {
		 alpha0[k] ~ dnorm(alpha0[k-1], eta0);
		 alpha[k] ~ dnorm(v*alpha0[k]+(1-v)*alpha[k-1], v*tau[p+1] + (1-v)*eta);
	   #Use for time-dependent gamma
	    #for(j in 1:q){
		 #gamma[k,j] ~ dnorm(gamma[k-1,j],xi[j])
	    #}
	  } 
		#Spike and Slab on Regression Coefficients
	  for (j in 1:p){
		 t[j] ~ dbern(p0);
		 slab[j] ~ dunif(l.slab,u.slab);
		 tau[j] <- (1-t[j])*spike[1] + t[j]*slab[j];
	  }
		#Spike and Slab on Baseline Hazard
		 t[p+1] ~ dbern(p0);
		 slab[p+1] ~ dunif(l.slab,u.slab);
		 tau[p+1] <- (1-t[p+1])*spike[2] + t[p+1]*slab[p+1];

		 eta0 <- 1/(sigma0*sigma0);
		 sigma0 ~ dunif(0.01,100);
		 eta <- 1/(sigma*sigma);
		 sigma ~ dunif(0.01,100);
}","OurModel.txt")


######################
#   Generic Inputs   #
######################
#Current Dataset
n = NA #Sample size of current dataset
y = rep(NA,n) #Follow-up times, vector of length n
delta = rep(NA,n) #Event Indicators, vector of length n
p = NA #Number of baseline covariates or # of columns in X below
X = matrix(NA,ncol=p,nrow=n) #Baseline Covariates in both datasets, n x p matrix
q = NA #Number of novel exposures or # of columns in Z below
Z = matrix(NA,ncol=q,nrow=n) #Exposures only in current dataset, n x q matrix

#Historical Dataset
n0 = NA #Sample size of historical dataset
y0 = rep(NA,n0) #Follow-up times, vector of length n0
delta0 = rep(NA,n0) #Event Indicators, vector of length n0
X0 = matrix(NA,ncol=p,nrow=n0) #Baseline Covariates in both datasets, n0 x p matrix

##################################
#   Simulated Heart Stent Data   #
##################################
dat <- read.csv("stent_sim.csv",header=TRUE)
attach(dat)
#Preprocess Covariates
 Xall = cbind(age_65to74,age_75plus)
 Zall = cbind(des)

#Current Dataset (2009 BMS/DES)
n = sum(year==2009) #Sample size of current dataset
y = fu[year==2009] #Follow-up times, vector of length n
delta = del[year==2009] #Event Indicators, vector of length n
p = dim(Xall)[2] #Number of baseline covariates or # of columns in X below
X = cbind(Xall[year==2009,]) #Baseline Covariates in both datasets, n x p matrix
q = dim(Zall)[2] #Number of novel exposures or # of columns in Z below
Z = cbind(Zall[year==2009,]) #Exposures only in current dataset, n x q matrix

#Historical Dataset (2008 BMS)
n0 = sum(year==2008) #Sample size of historical dataset
y0 = fu[year==2008] #Follow-up times, vector of length n0
delta0 = del[year==2008] #Event Indicators, vector of length n0
X0 = cbind(Xall[year==2008,]) #Baseline Covariates in both datasets, n0 x p matrix

##################################
#   Other Model Specifications   #
##################################
#"Spike and Slab" hyperparameters
l.slab=0.0001 #S_l (lower bound of slab) 
u.slab=2 #S_u (upper bound of slab)
p0=0.9 #p_0 (prior probability of being in the slab)
spike=c(200,200) #First Entry is Spike for Beta Regression Parameters, Second entry is Spike for baseline hazard (alpha parameter)

#Knot Placement/Location Calculations
K=max(5,min(floor(sum(delta)/8),20))
kappa=c(0,quantile(y[delta==1],seq(0,1,by=1/K))[-c(1,K+1)],1)

###################
#    JAGS Code    #
###################
model.data <- list("n0", "n", "p", "q", "K", "kappa", "y0", "delta0", "X0", "y", "delta", "X", "Z", "l.slab", "u.slab", "spike", "p0")
model.parameters <- c("beta", "gamma", "tau", "alpha")
#Use for constant gamma
model.initial.values <- function() {list (beta0=rnorm(p,0,3), beta=rnorm(p,0,3), gamma=rnorm(q,0,3), sigma0=dunif(1,0.01,100), sigma=dunif(1,0.01,100), alpha=rnorm(K,0,3), alpha0=rnorm(K,0,3))}
#Use for time-dependent gamma
#model.initial.values <- function() {list (beta0=rnorm(p,0,3), beta=rnorm(p,0,3), gamma=matrix(rnorm(q*K,0,3),ncol=q,nrow=K), sigma0=dunif(1,0.01,100), sigma=dunif(1,0.01,100), alpha=rnorm(K,0,3), alpha0=rnorm(K,0,3))}

#Run JAGS
model.fit <- jags(model.file="OurModel.txt", data=model.data, inits=model.initial.values,
               parameters=model.parameters, n.chains = 3, n.iter=502000, n.burnin=2000, n.thin=50)

#Compile Results and Compare to Cox PH
#Table 2 in Paper
#Hazard Ratio Estimates
est.comp = rbind(exp(c(model.fit$BUGSoutput$mean$beta,model.fit$BUGSoutput$mean$gamma)),
exp(coxph(Surv(y,delta)~X+Z)$coef),
exp(c(coxph(Surv(y0,delta0)~X0)$coef,NA)),
exp(coxph(Surv(c(y,y0),c(delta,delta0))~rbind(X,X0)+rbind(Z,cbind(rep(0,n0))))$coef))
rownames(est.comp) = c("Proposed Method","Current Cox PH","Historical Cox PH","Pooled Cox PH")
colnames(est.comp) = c(colnames(X),"des")
print(round(t(est.comp),3))

#Confidence/Credible Interval Estimates
curr.ph = coxph(Surv(y,delta)~X+Z)
round(summary(curr.ph)$conf.int[,c(3,4)],2)
hist.ph = coxph(Surv(y0,delta0)~X0)
round(summary(hist.ph)$conf.int[,c(3,4)],2)
pool.ph = coxph(Surv(c(y,y0),c(delta,delta0))~rbind(X,X0)+rbind(Z,cbind(rep(0,n0))))
round(summary(pool.ph)$conf.int[,c(3,4)],2)
round(t(apply(exp(model.fit$BUGSoutput$sims.matrix[,c(13,14,16)]),2,function(x) quantile(x,c(.025,.975)))),2)

#If using time-dependent gamma
#exp(model.fit$BUGSoutput$mean$gamma)
#round(t(apply(exp(model.fit$BUGSoutput$sims.matrix[,16:27]),2,function(x) quantile(x,c(.025,.975)))),2)